package ninja.thefirearchmage.games.fourtykcalculator;

import lombok.Getter;

import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Assumes a D6
 */
@Getter
public class Reroll {
    private final Set<Integer> rerollableValues;

    private Reroll(Set<Integer> rerollableValues) {
        this.rerollableValues = rerollableValues;
    }

    public static Reroll rerollOnOnes() {
        return new Reroll(Set.of(1));
    }

    public static Reroll rerollFailures(int targetHit) {
        var rerollableValues = IntStream.range(1, targetHit).boxed()
                .collect(Collectors.toSet());

        return new Reroll(rerollableValues);
    }

    public static Reroll rerollOnNot(Set<Integer> allowedValues) {
        var rerollableValues = IntStream.range(1,7).filter(i-> !allowedValues.contains(i))
                .boxed().collect(Collectors.toSet());

        return new Reroll(rerollableValues);
    }

    /**
     * From a D6 returns what proportion of rerollable values this reroll is.
     */
    public double getProbabilityNumerator() {
        return (double) rerollableValues.size() / 6;
    }
}
