package ninja.thefirearchmage.games.fourtykcalculator.utils.datastructures;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * Data structure to hold 2 different fields
 * @param <T> First field type
 * @param <R> Second field type
 */
@Getter
@ToString(of = {"_1", "_2"})
@EqualsAndHashCode(of = {"_1", "_2"}, callSuper = false)
public class Tuple2<T,R> extends Tuple {
    final public T _1;
    final public R _2;

    Tuple2(T _1, R _2) {
        this._1 = _1;
        this._2 = _2;
    }

    /**
     * Maps the first and second field with the given functions. Does not handle mapping errors.
     * @return a new Tuple with the mapped values
     * @param <U> the return type for the mapper
     */
    public <U> U map(BiFunction<T,R,U> tuppleMapper) {
        return tuppleMapper.apply(this._1, this._2);
    }

    /**
     * Returns a new tuple with each field being the result of applying the corresponding mapper function.
     * @param f1Mapper the mapping function to apply to the first field
     * @param f2Mapper the mapping function to apply to the second field
     * @return a new tuple with the mapper functions applied
     * @param <A> the new return type for the first field
     * @param <S> the new return type for the second field
     */
    public <A,S> Tuple2<A,S> map(Function<T,A> f1Mapper,
                                 Function<R,S> f2Mapper) {
        return Tuple.of(f1Mapper.apply(_1), f2Mapper.apply(_2));
    }

}
